from datetime import datetime

import pytest

from army_days.models import ConfigModel, DaysModel, EntryModel


@pytest.fixture
def standard_test_config() -> DaysModel:
    return DaysModel(
        config=ConfigModel(
            use_army_butt_days=False,
            show_completed=False,
        ),
        entries=[
            EntryModel(
                title="Item 1",
                date=datetime(2024, 12, 12),
            ),
            EntryModel(
                title="Item 2",
                date=datetime(2024, 8, 5),
            ),
            EntryModel(
                title="Item 3",
                date=datetime(2024, 8, 6),
            ),
            EntryModel(
                title="Item 4",
                date=datetime(2022, 4, 18),
            ),
            EntryModel(
                title="Item 5",
                date=datetime(2030, 1, 1),
            ),
            EntryModel(
                title="Item 6",
                date=datetime(2024, 8, 4),
            ),
        ],
    )



import os
import sys

import click
import yaml
import yaml.scanner
from pydantic_core import ValidationError

from .config import DEFAULT_CONFIG_FILES
from .core import compute_results, generate_default_configuration
from .models import DaysModel
from .output import output_events
from .utils import find_default_config_file


@click.command()
@click.version_option()
@click.option(
    "-f",
    "--filename",
    type=click.Path(),
    default="",
    help=f"configuration file; by default searches: {"\n".join(DEFAULT_CONFIG_FILES)}.",
)
@click.option(
    "-g",
    "--generate-sample",
    is_flag=True,
    show_default=True,
    default=False,
    help="generate sample data in yaml format (sends to stdout).",
)
def main(filename, generate_sample):
    if generate_sample:
        print(yaml.dump(generate_default_configuration().model_dump(mode="json")))
    else:
        config_filename = filename or find_default_config_file()
        if not config_filename or not os.path.exists(config_filename):
            sys.stderr.write(f"\nConfiguration file: '{config_filename}' not found.\n")
            sys.exit(1)
        with open(config_filename) as file:
            try:
                data = DaysModel(**yaml.load(file.read().replace("\t", " "), yaml.SafeLoader))
            except (yaml.scanner.ScannerError, yaml.error.YAMLError, ValidationError) as ex:
                sys.stderr.write(f"\nError parsing configuration file: {file.name} error: {ex}\n")
                sys.exit(1)
            output_events(compute_results(data))



import contextlib
import io
import json

from freezegun import freeze_time

from army_days.core import compute_results
from army_days.output import _output_color_events, _output_json_events


@freeze_time("2024-08-05T13:00:00")
def test_output_color_events(standard_test_config):
    standard_test_config.config.show_completed = True
    results = compute_results(standard_test_config)
    with io.StringIO() as buf:
        with contextlib.redirect_stdout(buf):
            _output_color_events(results)
        output_data = buf.getvalue()
    lines = output_data.split("\n")
    assert len(lines) == 8  # 8 lines of output including final \n
    assert "840 days since" in lines[1]
    assert "1 day since" in lines[2]
    assert "Today is " in lines[3]
    assert "1 day until" in lines[4]
    assert "129 days until" in lines[5]
    # testing army butt days
    standard_test_config.config.show_completed = True
    standard_test_config.config.use_army_butt_days = True
    results = compute_results(standard_test_config)
    with io.StringIO() as buf:
        with contextlib.redirect_stdout(buf):
            _output_color_events(results)
        output_data = buf.getvalue()
    lines = output_data.split("\n")
    assert len(lines) == 8  # 8 lines of output including final \n
    assert "840 days since" in lines[1]
    assert "1 day since" in lines[2]
    assert "Today is " in lines[3]
    assert "0 and a butt days until" in lines[4]
    assert "128 and a butt days until" in lines[5]


@freeze_time("2024-08-05T13:00:00")
def test_output_json_events(standard_test_config):
    results = compute_results(standard_test_config)
    with io.StringIO() as buf:
        with contextlib.redirect_stdout(buf):
            _output_json_events(results)
        output_data = buf.getvalue()
    assert output_data is not None
    data = json.loads(output_data)  # check that it's valid json
    assert data != {}
    assert data[0]["days"] == 0.0



import json
import sys

from .ansi_text import (
    RESET,
    UNDERLINE,
    rgb_background,
    rgb_foreground,
)
from .models import ComputedEventModel

BG_BLACK = rgb_background(0, 0, 0)
BG_DARK_GRAY = rgb_background(30, 30, 30)
FG_YELLOW = rgb_foreground(255, 255, 0)

HEADING = " Days"


def _output_json_events(events: list[ComputedEventModel]):
    print(json.dumps([json.loads(event.model_dump_json()) for event in events]))


def _output_color_events(events: list[ComputedEventModel]):
    lines = []
    longest_line = HEADING
    for event in events:
        string_days = (
            f"{abs(event.days):.0f}" if float(int(event.days)) == event.days else f"{abs(event.days):.0f} and a butt"
        )
        match event.days:
            case 0:
                string_days = "Today is"
            case 1:
                string_days += " day until"
            case -1:
                string_days += " day since"
            case days if days > 0:
                string_days += " days until"
            case days if days < 0:
                string_days += " days since"
        line = f"{string_days} {event.title}."
        if len(line) > len(longest_line):
            longest_line = line
        lines.append(line)

    max_len = len(longest_line)
    sys.stdout.write(f"{FG_YELLOW}{UNDERLINE}{HEADING:<{max_len}}{RESET}\n")
    for i, line in enumerate(lines):
        bg = BG_BLACK if i % 2 != 0 else BG_DARK_GRAY
        sys.stdout.write(f"{bg}{line:<{max_len}}{RESET}")
        sys.stdout.write("\n")


def output_events(events: list[ComputedEventModel]):
    if sys.stdout.isatty():
        _output_color_events(events)
    else:
        _output_json_events(events)



import os

from ..config import DEFAULT_CONFIG_FILES


def find_default_config_file() -> str:
    for filename in DEFAULT_CONFIG_FILES:
        if os.path.exists(os.path.expanduser(filename)):
            return os.path.expanduser(filename)
    return ""



from datetime import datetime

from .models import ComputedEventModel, ConfigModel, DaysModel, EntryModel


def compute_results(days: DaysModel) -> list[ComputedEventModel]:
    results = []
    now = datetime.now()
    current = datetime(now.year, now.month, now.day)
    for entry in days.entries:
        tmp_date = entry.date.astimezone()  # force it to local time
        event_date = datetime(tmp_date.year, tmp_date.month, tmp_date.day)
        time_delta = (event_date - current).days
        new_computed_event = ComputedEventModel(title=entry.title, date=tmp_date, days=time_delta)
        if time_delta > 0 and days.config.use_army_butt_days and now.hour >= 12:
            new_computed_event.days -= 0.5
        if time_delta >= 0 or days.config.show_completed:
            results.append(new_computed_event)
    results.sort(key=lambda x: x.date)
    return results


def generate_default_configuration() -> DaysModel:
    now = datetime.now()
    if now.month == 2 and now.day == 29:
        new_date = datetime(now.year + 1, 3, 1)
    else:
        new_date = datetime(now.year + 1, now.month, now.day)
    return DaysModel(
        config=ConfigModel(),
        entries=[
            EntryModel(title="your one year anniversary of using army-days", date=new_date),
        ],
    )



from datetime import datetime

from freezegun import freeze_time

from army_days.core import compute_results, generate_default_configuration


@freeze_time("2024-08-05T13:00:00")
def test_compute_results_standard(standard_test_config):
    results = compute_results(standard_test_config)
    assert results is not None
    assert results[0].title == "Item 2"  # verify sorting works
    assert len(results) == 4  # verify past events are excluded
    assert results[2].days == 129.0  # verify that we're *not* using army butt days
    # switch settings...
    standard_test_config.config.use_army_butt_days = True
    standard_test_config.config.show_completed = True
    results = compute_results(standard_test_config)
    assert results is not None
    assert results[0].title == "Item 4"  # verify sorting and showcompleted
    assert len(results) == 6  # verify past events are included
    assert results[0].days == -840.0
    assert results[4].days == 128.5  # army butt days now in use


@freeze_time("2024-08-05T13:00:00")
def test_generate_default_configuration():
    config = generate_default_configuration()
    assert config is not None
    assert config.config.use_army_butt_days is False
    assert config.config.show_completed is False
    assert len(config.entries) == 1
    assert config.entries[0].date == datetime(2025, 8, 5)


@freeze_time("2024-02-29T13:00:00")  # LEAP DAY
def test_generate_default_configuration_leap_day():
    config = generate_default_configuration()
    assert config is not None
    assert config.config.use_army_butt_days is False
    assert config.config.show_completed is False
    assert len(config.entries) == 1
    assert config.entries[0].date == datetime(2025, 3, 1)



from datetime import datetime

from pydantic import BaseModel, Field


class ConfigModel(BaseModel):
    use_army_butt_days: bool = Field(default=False, alias="useArmyButtDays")
    show_completed: bool = Field(default=False, alias="showCompleted")


class EntryModel(BaseModel):
    title: str
    date: datetime


class DaysModel(BaseModel):
    config: ConfigModel
    entries: list[EntryModel]


class ComputedEventModel(EntryModel):
    days: float



from datetime import datetime

from army_days.models import ConfigModel, DaysModel, EntryModel


def test_config_model():
    blank_model = ConfigModel()
    assert blank_model is not None
    assert blank_model.use_army_butt_days is False
    assert blank_model.show_completed is False


def test_days_model():
    blank_model = DaysModel(config=ConfigModel(), entries=[])
    assert blank_model is not None


def test_entry_model():
    simple_model = EntryModel(title="nothing", date=datetime(1, 1, 1))
    assert simple_model is not None
